-- ABSMod.lua
-- author: pavll
-- date: 19.03.2020
-- Copyright 2020

ABSMod = {};
local ABSMod_mt = Class(ABSMod)

ABSMod.ACTIVE_DEVELOPMENT_MODE = false
ABSMod.ENABLE_CONSOLE_COMMANDS_INTERFACE = true
ABSMod.ENABLE_ABS = true

function ABSMod:new(isServer, isClient, modDir, name)
	local spec = self.spec_absmod

	if g_absMod ~= nil then
        return
    end

    local self = {}
    setmetatable(self, ABSMod_mt)

    self.isServer = isServer
    self.isClient = isClient

    self.isDeveloper = false -- only for pavll

    self.baseDirectory = modDir

    return self
end;

function ABSMod:onLoad(savegame)
	local spec = self.spec_absmod
	
	if self.isClient then
		resetSteers(self)
	end
end;

function ABSMod:onUpdate(dt, isSelected)
	if self.isClient and ABSMod.ENABLE_ABS then
		local spec = self.spec_absmod
		
		--															for avoiding error with steering
		if self:getIsActive() and self.spec_drivable ~= nil and self.spec_drivable.steeringWheel ~= nil then
			local _, steeringRotationY, _ = getRotation(self.spec_drivable.steeringWheel.node)

		    if self.spec_drivable.hasSteerToTurnRightDirection or 
		    	self.spec_drivable.hasSteerToTurnLeftDirection then
		    	local permAutoStopTurnLights = false

		    	-- mod checks for permmission if it could automaticly turn off this lights 
		    	-- if the steering wheel is nearly centered 
		    	if self.spec_drivable.hasSteerToTurnRightDirection then
		    		permAutoStopTurnLights = steeringRotationY >= -0.001
		    	elseif self.spec_drivable.hasSteerToTurnLeftDirection then
		    		permAutoStopTurnLights = steeringRotationY <= 0.001
		    	end

		    	if permAutoStopTurnLights then
		    		self:setTurnLightState(Lights.TURNLIGHT_OFF)

			    	resetSteers(self)
			    end
			elseif self.spec_lights.turnLightState == Lights.TURNLIGHT_RIGHT then
				self.spec_drivable.hasSteerToTurnRightDirection = steeringRotationY < -0.1;

			elseif self.spec_lights.turnLightState == Lights.TURNLIGHT_LEFT then
				self.spec_drivable.hasSteerToTurnLeftDirection = steeringRotationY > 0.1;
			end
		end
	end
end;	

function ABSMod:loadMap(name)
	-- linking Drivable events with this customs
	Drivable.onLoad = Utils.prependedFunction(Drivable.onLoad, ABSMod.onLoad)
	Drivable.onUpdate = Utils.prependedFunction(Drivable.onUpdate, ABSMod.onUpdate)
end;

function ABSMod:deleteMap()
	if self.debugConsoleCommands ~= nil then
		self.debugConsoleCommands:delete()
	end
end;

function absModInit(name, a)
	if name == nil or a == nil then
		return
	end

	if g_absMod == nil and (name == "FS19_AutoBlinkerStopMod" or name == "FS19_AutoBlinkerStopMod_update") and a:len() == 5 then
		local modDir = g_currentModDirectory
		local absMod = ABSMod:new(g_server ~=nil, g_dedicatedServerInfo == nil, modDir, name)

		if absMod ~= nil then
			getfenv(0)["g_absMod"] = absMod

			if absMod.isClient then
				if ABSMod.ACTIVE_DEVELOPMENT_MODE then
					absMod.isDeveloper = true

					if absMod.isServer then
						-- something for development like debuging stuff
					end
				end
				
				if absMod.ENABLE_CONSOLE_COMMANDS_INTERFACE or absMod.isDeveloper then
					source(modDir .. "src/DebugConsoleCommands.lua")
					absMod.debugConsoleCommands = DebugConsoleCommands:new(absMod)
				end
			end

			-- space for implementing some lua gui helper scripts in the future :)
				---
				---

			-- Add events for testing on dedicated servers also if some commands are server only.
				---
				---

			-- stuff like register some action events like that
			--[[

			Player.registerActionEvents = Utils.appendedFunction(Player.registerActionEvents, 
				registerPlayerActionEvents)
	        Player.removeActionEvents = Utils.appendedFunction(Player.removeActionEvents, 
	        	removePlayerActionEvents)	]]

	        print(" Loaded Automatic Blinker(turn lights) Stop Modification successfully ")
		end
	else
		print("Error[ABSMod]: There is a problem(Code: 1x0), please redownload and reinstall this mod!")
	end
end;

function resetSteers(modSpec)
	-- reset drivable vehicle steers
	modSpec.spec_drivable.hasSteerToTurnLeftDirection = false
	modSpec.spec_drivable.hasSteerToTurnRightDirection = false
end; 

absModInit(g_currentModName, g_modManager:getModByName(g_currentModName).author)
addModEventListener(ABSMod)